#!/usr/bin/env bash

###############################################################################
# FILE:         appoptics-host-agent-config.sh
# VERSION:      1.0.0
# DESCRIPTION:  Agent configuration script for multiple OS/Distributions
# BUGS:         support@appoptics.com
# COPYRIGHT:    (c) 2017 Librato
# LICENSE:      Apache 2.0; http://www.apache.org/licenses/LICENSE-2.0
# ORGANIZATION: http://appoptics.com
#
# NOTICE: Some code borrowed from bootstrap-salt.sh (License: Apache 2.0)
#         https://github.com/saltstack/salt-bootstrap/blob/develop/bootstrap-salt.sh
###############################################################################
_SD_SCRIPT_VERSION="1.0.0"

DEFAULT_INSTALL_DIR=/opt/appoptics
INSTALL_DIR=${INSTALL_DIR:-${DEFAULT_INSTALL_DIR}}
DEFAULT_PLUGINS_DIR=${INSTALL_DIR}/bin
DEFAULT_TASKS_DIR=${INSTALL_DIR}/etc/tasks.d
DEFAULT_PLUGINS_CONF_DIR=${INSTALL_DIR}/etc/plugins.d
DEFAULT_AUTOLOAD_DIR=${INSTALL_DIR}/autoload
APPOPTICS_CONF="${INSTALL_DIR}/etc/config.yaml"
SNAP_SERVICE_NAME=appoptics-snapteld
AGENT_DISPLAY_NAME="AppOptics Host Agent"
PACKAGE_NAME=appoptics-snaptel
DEFAULT_TASK_EXTENSION=yaml

# Settings for the user used to run the daemon
APPOPTICS_USERNAME=appoptics
APPOPTICS_GROUPNAME=appoptics

SYSTEMCTL_BIN=`command -v systemctl 2>/dev/null`
CHKCONFIG_BIN=`command -v chkconfig 2>/dev/null`
SERVICE_BIN=`command -v service 2>/dev/null`
CURL_BIN=`command -v curl 2>/dev/null`

# Return error codes
readonly SUCCESS=0
readonly FAILURE=1
readonly FAILURE_SERVICE_NOT_RUNNING=2
readonly FAILURE_SERVICE_UNABLE_TO_STOP=3
readonly FAILURE_NOT_ENOUGH_RIGHTS=4
readonly FAILURE_MISSING_EXECUTABLE=5
readonly FAILURE_MISSING_FILE=6

# truth values
__IS_TRUE=1
__IS_FALSE=0

_SD_NO_COLOR=$__IS_FALSE
_CURL_ARGS=""

# --------------------------------------------
# -------------- Begin Helper Functions ------
# --------------------------------------------

# Check for "--no-color" flag in opts
printf "%s" "$@" | grep -q -- '--no-color' >/dev/null 2>&1
if [ $? -eq 0 ]; then
    _SD_NO_COLOR=$__IS_TRUE
fi

# Current directory used to run the script
ROOT_DIR=`pwd`
INSTALLER_LOG_FILE="${ROOT_DIR}/${0}.log"

declare -A ubuntu_codename
ubuntu_codename["12.04"]="precise"
ubuntu_codename["14.04"]="trusty"
ubuntu_codename["15.04"]="vivid"
ubuntu_codename["16.04"]="xenial"
ubuntu_codename["17.04"]="zesty"

declare -A amazon_codename
amazon_codename["2016.03"]="6"
amazon_codename["2016.09"]="6"
amazon_codename["2017.03"]="7"

declare -A default_plugins_long_names
default_plugins_long_names["system"]="snap-plugin-collector-aosystem"
default_plugins_long_names["file"]="snap-plugin-publisher-file"
default_plugins_long_names["docker"]="snap-plugin-collector-aodocker"
default_plugins_long_names["apache"]="snap-plugin-collector-aoapache"
default_plugins_long_names["mongodb"]="snap-plugin-collector-aomongodb"
default_plugins_long_names["mysql"]="snap-plugin-collector-aomysql"
default_plugins_long_names["nginxplus"]="snap-plugin-collector-aonginxplus"

declare -A default_plugins_tasks
default_plugins_tasks["system"]="task-aosystem-publish-appoptics"
default_plugins_tasks["file"]="task-aosystem-publish-file"
default_plugins_tasks["docker"]="task-docker-publish-appoptics"
default_plugins_tasks["apache"]="task-apache-publish-appoptics"
default_plugins_tasks["mongodb"]="task-mongodb-publish-appoptics"
default_plugins_tasks["mysql"]="task-mysql-publish-appoptics"
default_plugins_tasks["nginxplus"]="task-nginxplus-publish-appoptics"

declare -A default_plugins_configuration_file
default_plugins_configuration_file["system"]="config.yaml"
default_plugins_configuration_file["file"]="file.yaml"
default_plugins_configuration_file["docker"]="docker.yaml"
default_plugins_configuration_file["apache"]="apache.yaml"
default_plugins_configuration_file["mongodb"]="mongodb.yaml"
default_plugins_configuration_file["mysql"]="mysql.yaml"
default_plugins_configuration_file["nginxplus"]="nginxplus.yaml"

_COLORS=${BS_COLORS:-$(tput colors 2>/dev/null || echo 0)}
__detect_color_support() 
{
    if [ $? -eq 0 ] && [ "$_COLORS" -gt 2 ] && [ $_SD_NO_COLOR -eq $__IS_FALSE ]; then
        RC="\033[1;31m"
        GC="\033[1;32m"
        BC="\033[1;34m"
        YC="\033[1;33m"
        EC="\033[0m"
    else
        RC=""
        GC=""
        BC=""
        YC=""
        EC=""
    fi
}
__detect_color_support 

# Override built-in echo for consistency. Has newline at end
echo () (
fmt=%s end=\\n IFS=" "

while [ $# -gt 1 ] ; do
    case "$1" in
        [!-]*|-*[!ne]*) break ;;
        *ne*|*en*) fmt=%b end= ;;
        *n*) end= ;;
        *e*) fmt=%b ;;
    esac
    shift
done

printf "$fmt$end" "$*"
)

echored() 
{
    printf "${RC}%s${EC}\n" "$*";
}

echogreen() 
{
    printf "${GC}%s${EC}\n" "$*";
}

echoblue()
{
    printf "${BC}%s${EC}\n" "$*";
}

echoyellow()
{
    printf "${YC}%s${EC}\n" "$*";
}

exists()
{
  command -v "$1" >/dev/null 2>&1
}

now()
{
  echo $(date +'%Y-%m-%d-%H_%M_%S')
}

echoerror() 
{
    printf "${RC} * ERROR${EC}: %s\n" "$*" 1>&2;
}

echoinfo ()
{
    printf "${GC} *  INFO${EC}: %s\n" "$*";
}

echowarn ()
{
  printf "${YC} *  WARN${EC}: %s\n" "$*";
}

# Print debug to STDOUT
echodebug()
{
    if [ "$_ECHO_DEBUG" = "$__IS_TRUE" ]; then
        printf "${BC} * DEBUG${EC}: %s\n" "$*";
    fi
}

failed ()
{
  error_code=${1}
  echored "###################################################"
  echored "An error occured during the configuration of ${AGENT_DISPLAY_NAME}."
  echored "Please contact AppOptics customer service for support at support@appoptics.com."
  echored "###################################################"
}

owner_of_file()
{
    if [ -e $1 ]; then
        ls -ld $1 | awk '{print $3}'
    fi
}

group_of_file()
{
    if [ -e $1 ]; then
        ls -ld $1 | awk '{print $4}'
    fi
}

# Retrieves a URL and writes it to a given path
__fetch_url() 
{
    echodebug "Downloading $2 into $1 ..."
    curl $_CURL_ARGS -s -o "$1" "$2" >/dev/null 2>&1 ||
        wget $_WGET_ARGS -q -O "$1" "$2" >/dev/null 2>&1 ||
            fetch $_FETCH_ARGS -q -o "$1" "$2" >/dev/null 2>&1 ||
                fetch -q -o "$1" "$2" >/dev/null 2>&1           # Pre FreeBSD 10
}

# Discover hardware information
__gather_hardware_info()
{
    if [ -f /proc/cpuinfo ]; then
        CPU_VENDOR_ID=$(awk '/vendor_id|Processor/ {sub(/-.*$/,"",$3); print $3; exit}' /proc/cpuinfo )
    elif [ -f /usr/bin/kstat ]; then
        # SmartOS.
        # Solaris!?
        # This has only been tested for a GenuineIntel CPU
        CPU_VENDOR_ID=$(/usr/bin/kstat -p cpu_info:0:cpu_info0:vendor_id | awk '{print $2}')
    else
        CPU_VENDOR_ID=$( sysctl -n hw.model )
    fi
    # shellcheck disable=SC2034
    CPU_VENDOR_ID_L=$( echo "$CPU_VENDOR_ID" | tr '[:upper:]' '[:lower:]' )
    CPU_ARCH=$(uname -m 2>/dev/null || uname -p 2>/dev/null || echo "unknown")
    CPU_ARCH_L=$( echo "$CPU_ARCH" | tr '[:upper:]' '[:lower:]' )

}
__gather_hardware_info

# DESCRIPTION:  Discover operating system information
__gather_os_info() 
{
    OS_NAME=$(uname -s 2>/dev/null)
    OS_NAME_L=$( echo "$OS_NAME" | tr '[:upper:]' '[:lower:]' )
    OS_VERSION=$(uname -r)
    OS_VERSION_L=$( echo "$OS_VERSION" | tr '[:upper:]' '[:lower:]' )
}
__gather_os_info

#   Parse version strings ignoring the revision.
#   MAJOR.MINOR.REVISION becomes MAJOR.MINOR
__parse_version_string()
{
    VERSION_STRING="$1"
    PARSED_VERSION=$(
        echo "$VERSION_STRING" |
        sed -e 's/^/#/' \
            -e 's/^#[^0-9]*\([0-9][0-9]*\.[0-9][0-9]*\)\(\.[0-9][0-9]*\).*$/\1/' \
            -e 's/^#[^0-9]*\([0-9][0-9]*\.[0-9][0-9]*\).*$/\1/' \
            -e 's/^#[^0-9]*\([0-9][0-9]*\).*$/\1/' \
            -e 's/^#.*$//'
    )
    echo "$PARSED_VERSION"
}

#   Strip single or double quotes from the provided string.
__unquote_string() 
{
    echo "${@}" | sed "s/^\([\"']\)\(.*\)\1\$/\2/g"
}

#   Convert CamelCased strings to Camel_Cased
__camelcase_split() {
    echo "${@}" | sed -r 's/([^A-Z-])([A-Z])/\1 \2/g'
}

#   DESCRIPTION:  Strip duplicate strings
__strip_duplicates()
{
    echo "${@}" | tr -s '[:space:]' '\n' | awk '!x[$0]++'
}

__distro_packager_info() 
{
    case "${DISTRO_NAME_L}" in
        "ubuntu"|"debian")
            DISTRO_PACKAGE_TYPE="deb"
            DISTRO_PACKAGE_MANAGER="dpkg"
            ;;
        red_hat*|"centos"|"scientific_linux"|"oracle_linux")
            DISTRO_PACKAGE_TYPE="rpm"
            DISTRO_PACKAGE_MANAGER="yum"
            ;;
        "amazon_linux_ami")
            DISTRO_PACKAGE_TYPE="rpm"
            DISTRO_PACKAGE_MANAGER="yum"
            ;;
        "fedora")
            DISTRO_PACKAGE_TYPE="rpm"
            DISTRO_PACKAGE_MANAGER="yum"
            ;;
        *)
            DISTRO_PACKAGE_TYPE=""
            echodebug "Could not determine DISTRO_PACKAGE_MANAGER in function ${0}"
            ;;
    esac
}

__sort_release_files()
{
    KNOWN_RELEASE_FILES=$(echo "(arch|centos|debian|ubuntu|fedora|redhat|suse|\
        mandrake|mandriva|gentoo|slackware|turbolinux|unitedlinux|lsb|system|\
        oracle|os)(-|_)(release|version)" | sed -r 's:[[:space:]]::g')
    primary_release_files=""
    secondary_release_files=""
    # Sort know VS un-known files first
    for release_file in $(echo "${@}" | sed -r 's:[[:space:]]:\n:g' | sort --unique --ignore-case); do
        match=$(echo "$release_file" | egrep -i "${KNOWN_RELEASE_FILES}")
        if [ "${match}" != "" ]; then
            primary_release_files="${primary_release_files} ${release_file}"
        else
            secondary_release_files="${secondary_release_files} ${release_file}"
        fi
    done
    # Now let's sort by know files importance, max important goes last in the max_prio list
    max_prio="redhat-release centos-release oracle-release"
    for entry in $max_prio; do
        if [ "$(echo "${primary_release_files}" | grep "$entry")" != "" ]; then
            primary_release_files=$(echo "${primary_release_files}" | sed -e "s:\(.*\)\($entry\)\(.*\):\2 \1 \3:g")
        fi
    done
    # Now, least important goes last in the min_prio list
    min_prio="lsb-release"
    for entry in $min_prio; do
        if [ "$(echo "${primary_release_files}" | grep "$entry")" != "" ]; then
            primary_release_files=$(echo "${primary_release_files}" | sed -e "s:\(.*\)\($entry\)\(.*\):\1 \3 \2:g")
        fi
    done
    # Echo the results collapsing multiple white-space into a single white-space
    echo "${primary_release_files} ${secondary_release_files}" | sed -r 's:[[:space:]]+:\n:g'
}

#   DESCRIPTION:  Discover Linux system information
__gather_linux_system_info()
{
    DISTRO_NAME=""
    DISTRO_VERSION=""
    # Let's test if the lsb_release binary is available
    rv=$(lsb_release >/dev/null 2>&1)
    if [ $? -eq 0 ]; then
        DISTRO_NAME=$(lsb_release -si)
        if [ "${DISTRO_NAME}" = "Scientific" ]; then
            DISTRO_NAME="Scientific Linux"
        elif [ "$(echo "$DISTRO_NAME" | grep RedHat)" != "" ]; then
            # Let's convert CamelCase to Camel Case
            DISTRO_NAME=$(__camelcase_split "$DISTRO_NAME")
        elif [ "${DISTRO_NAME}" = "openSUSE project" ]; then
            # lsb_release -si returns "openSUSE project" on openSUSE 12.3
            DISTRO_NAME="opensuse"
        elif [ "${DISTRO_NAME}" = "SUSE LINUX" ]; then
            if [ "$(lsb_release -sd | grep -i opensuse)" != "" ]; then
                # openSUSE 12.2 reports SUSE LINUX on lsb_release -si
                DISTRO_NAME="opensuse"
            else
                # lsb_release -si returns "SUSE LINUX" on SLES 11 SP3
                DISTRO_NAME="suse"
            fi
        elif [ "${DISTRO_NAME}" = "EnterpriseEnterpriseServer" ]; then
            # This the Oracle Linux Enterprise ID before ORACLE LINUX 5 UPDATE 3
            DISTRO_NAME="Oracle Linux"
        elif [ "${DISTRO_NAME}" = "OracleServer" ]; then
            # This the Oracle Linux Server 6.5
            DISTRO_NAME="Oracle Linux"
        elif [ "${DISTRO_NAME}" = "AmazonAMI" ]; then
            DISTRO_NAME="Amazon Linux AMI"
        elif [ "${DISTRO_NAME}" = "Arch" ]; then
            DISTRO_NAME="Arch Linux"
            return
        fi
        rv=$(lsb_release -sr)
        [ "${rv}" != "" ] && DISTRO_VERSION=$(__parse_version_string "$rv")
    elif [ -f /etc/lsb-release ]; then
        # We don't have the lsb_release binary, though, we do have the file it parses
        DISTRO_NAME=$(grep DISTRIB_ID /etc/lsb-release | sed -e 's/.*=//')
        rv=$(grep DISTRIB_RELEASE /etc/lsb-release | sed -e 's/.*=//')
        [ "${rv}" != "" ] && DISTRO_VERSION=$(__parse_version_string "$rv")
    fi
    if [ "$DISTRO_NAME" != "" ] && [ "$DISTRO_VERSION" != "" ]; then
        # We already have the distribution name and version
        return
    fi
    for rsource in $(__sort_release_files "$(
            cd /etc && /bin/ls *[_-]release *[_-]version 2>/dev/null | env -i sort | \
            sed -e '/^redhat-release$/d' -e '/^lsb-release$/d'; \
            echo redhat-release lsb-release
            )"); do
        [ -L "/etc/${rsource}" ] && continue        # Don't follow symlinks
        [ ! -f "/etc/${rsource}" ] && continue      # Does not exist
        n=$(echo "${rsource}" | sed -e 's/[_-]release$//' -e 's/[_-]version$//')
        shortname=$(echo "${n}" | tr '[:upper:]' '[:lower:]')
        rv=$( (grep VERSION "/etc/${rsource}"; cat "/etc/${rsource}") | grep '[0-9]' | sed -e 'q' )
        [ "${rv}" = "" ] && [ "$shortname" != "arch" ] && continue  # There's no version information. Continue to next rsource
        v=$(__parse_version_string "$rv")
        case $shortname in
            redhat             )
                if [ "$(egrep 'CentOS' /etc/${rsource})" != "" ]; then
                    n="CentOS"
                elif [ "$(egrep 'Scientific' /etc/${rsource})" != "" ]; then
                    n="Scientific Linux"
                elif [ "$(egrep 'Red Hat Enterprise Linux' /etc/${rsource})" != "" ]; then
                    n="Red Hat Enterprise Linux"
                else
                    n="Red Hat Linux"
                fi
                ;;
            arch               ) n="Arch Linux"     ;;
            centos             ) n="CentOS"         ;;
            debian             ) n="Debian"         ;;
            ubuntu             ) n="Ubuntu"         ;;
            fedora             ) n="Fedora"         ;;
            suse               ) n="SUSE"           ;;
            mandrake*|mandriva ) n="Mandriva"       ;;
            gentoo             ) n="Gentoo"         ;;
            slackware          ) n="Slackware"      ;;
            turbolinux         ) n="TurboLinux"     ;;
            unitedlinux        ) n="UnitedLinux"    ;;
            oracle             ) n="Oracle Linux"   ;;
            system             )
                while read -r line; do
                    [ "${n}x" != "systemx" ] && break
                    case "$line" in
                        *Amazon*Linux*AMI*)
                            n="Amazon Linux AMI"
                            break
                    esac
                done < "/etc/${rsource}"
                ;;
            os                 )
                nn="$(__unquote_string "$(grep '^ID=' /etc/os-release | sed -e 's/^ID=\(.*\)$/\1/g')")"
                rv="$(__unquote_string "$(grep '^VERSION_ID=' /etc/os-release | sed -e 's/^VERSION_ID=\(.*\)$/\1/g')")"
                [ "${rv}" != "" ] && v=$(__parse_version_string "$rv") || v=""
                case $(echo "${nn}" | tr '[:upper:]' '[:lower:]') in
                    amzn        )
                        # Amazon AMI's after 2014.9 match here
                        n="Amazon Linux AMI"
                        ;;
                    arch        )
                        n="Arch Linux"
                        v=""  # Arch Linux does not provide a version.
                        ;;
                    debian      )
                        n="Debian"
                        if [ "${v}" = "" ]; then
                            if [ "$(cat /etc/debian_version)" = "wheezy/sid" ]; then
                                # I've found an EC2 wheezy image which did not tell its version
                                v=$(__parse_version_string "7.0")
                            elif [ "$(cat /etc/debian_version)" = "jessie/sid" ]; then
                                # Let's start detecting the upcoming Debian 8 (Jessie)
                                v=$(__parse_version_string "8.0")
                            fi
                        else
                            echowarn "Unable to parse the Debian Version"
                        fi
                        ;;
                    *           )
                        n=${nn}
                        ;;
                esac
                ;;
            *                  ) n="${n}"           ;
        esac
        DISTRO_NAME=$n
        DISTRO_VERSION=$v
        break
    done
}

__gather_system_info()
{
    case ${OS_NAME_L} in
        linux )
            __gather_linux_system_info
            ;;
        * )
            echoerror "${OS_NAME} not supported.";
            exit 1
            ;;
    esac
}

#   Check for end of life distribution versions
__check_end_of_life_versions()
{
    case "${DISTRO_NAME_L}" in
        debian)
            # Debian versions bellow 6 are not supported
            if [ "$DISTRO_MAJOR_VERSION" -lt 6 ]; then
                echoerror "End of life distributions are not supported."
                echoerror "Please consider upgrading to the next stable. See:"
                echoerror "    https://wiki.debian.org/DebianReleases"
                exit 1
            fi
            ;;
        ubuntu)
            # Ubuntu versions not supported
            #
            #  < 10
            #  = 10.10
            #  = 11.04
            #  = 11.10
            if ([ "$DISTRO_MAJOR_VERSION" -eq 10 ] && [ "$DISTRO_MINOR_VERSION" -eq 10 ]) || \
               ([ "$DISTRO_MAJOR_VERSION" -eq 11 ] && [ "$DISTRO_MINOR_VERSION" -eq 04 ]) || \
               ([ "$DISTRO_MAJOR_VERSION" -eq 11 ] && [ "$DISTRO_MINOR_VERSION" -eq 10 ]) || \
               [ "$DISTRO_MAJOR_VERSION" -lt 10 ]; then
                echoerror "End of life distributions are not supported."
                echoerror "Please consider upgrading to the next stable. See:"
                echoerror "    https://wiki.ubuntu.com/Releases"
                exit 1
            fi
            ;;
        fedora)
            # Fedora lower than 18 are no longer supported
            if [ "$DISTRO_MAJOR_VERSION" -lt 18 ]; then
                echoerror "End of life distributions are not supported."
                echoerror "Please consider upgrading to the next stable. See:"
                echoerror "    https://fedoraproject.org/wiki/Releases"
                exit 1
            fi
            ;;
        centos)
            # CentOS versions lower than 6 are no longer supported
            if [ "$DISTRO_MAJOR_VERSION" -lt 6 ]; then
                echoerror "End of life distributions are not supported."
                echoerror "Please consider upgrading to the next stable. See:"
                echoerror "    http://wiki.centos.org/Download"
                exit 1
            fi
            ;;
        red_hat*linux)
            # Red Hat (Enterprise) Linux versions lower than 5 are no longer supported
            if [ "$DISTRO_MAJOR_VERSION" -lt 5 ]; then
                echoerror "End of life distributions are not supported."
                echoerror "Please consider upgrading to the next stable. See:"
                echoerror "    https://access.redhat.com/support/policy/updates/errata/"
                exit 1
            fi
            ;;
        *)
            ;;
    esac
}

__package_status()
{
    if [ "${1}" = "" ]; then
        echo ""
    else
        case "${DISTRO_PACKAGE_MANAGER}" in
            "dpkg")
                dpkg -s ${1} 2>/dev/null| grep Status | sed 's/^Status: //'
                ;;
            "yum")
                yum list installed ${1} 2>/dev/null | grep "^${1}" | awk '{print $NF}'
                ;;
        esac
    fi
}

__is_package_installed()
{
    __package_state="unknown"
    case "${DISTRO_PACKAGE_MANAGER}" in
        "yum")
            $(__package_status ${1} | grep -q -E "(^installed$|^@)")
            if [ $? -eq 0 ]; then
                __package_state="installed"
            fi
            ;;
        "dpkg")
            $(__package_status ${1} | grep -q " installed$")
            if [ $? -eq 0 ]; then
                __package_state="installed"
            fi
            ;;
    esac
    if [ "$__package_state" = "installed" ]; then
        echo $__IS_TRUE
    else
        echo $__IS_FALSE
    fi
}

__create_temp_directory()
{
    _SD_TMP_DIR=$(mktemp -d -t appoptics.XXXX)
    if [ $? -ne 0 ]; then
        echoerror "Could not create temporary directory"
        exit 1
    fi
    echodebug "Created temp directory: $_SD_TMP_DIR"
}

# --------------------------------------------
# -------------- End Helper Functions --------
# --------------------------------------------

detect_operating_system()
{
    echodebug "Detecting operating system ..."
    __gather_system_info
    DISTRO_NAME_L=$(echo "$DISTRO_NAME" | tr '[:upper:]' '[:lower:]' | sed 's/[^a-zA-Z0-9_ ]//g' | sed -re 's/([[:space:]])+/_/g')
    DISTRO_MAJOR_VERSION=$(echo "$DISTRO_VERSION" | sed 's/^\([0-9]*\).*/\1/g')
    DISTRO_MINOR_VERSION=$(echo "$DISTRO_VERSION" | sed 's/^\([0-9]*\).\([0-9]*\).*/\2/g')
    __distro_packager_info
    __check_end_of_life_versions
}

distro_not_supported()
{
    echodebug "Using DISTRO_NAME_L: ${DISTRO_NAME_L}"
    echoerror "Sorry, we don't currently support ${DISTRO_NAME} ${DISTRO_VERSION}."
    exit 1
}

cpu_arch_not_supported()
{
    echoerror "Sorry, we don't currently support your CPU Architecture, ${CPU_ARCH}."
    exit 1
}

print_system_info() 
{
    echodebug "System Information:"
    echodebug "  CPU:          ${CPU_VENDOR_ID}"
    echodebug "  CPU Arch:     ${CPU_ARCH}"
    echodebug "  OS Name:      ${OS_NAME}"
    echodebug "  OS Version:   ${OS_VERSION}"
    echodebug "  Distribution: ${DISTRO_NAME} ${DISTRO_VERSION}"
}

check_distro_supported() 
{
    print_system_info
    case ${CPU_ARCH_L} in
        i[3456]86|x86_64)
            ;;
        *)
            cpu_arch_not_supported
            ;;
    esac

    case "${DISTRO_NAME_L}" in
        "ubuntu")
            if [ ${DISTRO_MAJOR_VERSION} -lt 12 ] || [ ${DISTRO_MAJOR_VERSION} -gt 16 ]; then
                distro_not_supported
            fi
            ;;
        "debian")
            if [ ${DISTRO_MAJOR_VERSION} -lt 7 ] || [ ${DISTRO_MAJOR_VERSION} -gt 8 ]; then
                distro_not_supported
            fi
            ;;
        red_hat*|"centos")
            if [ ${DISTRO_MAJOR_VERSION} -lt 6 ]; then
                distro_not_supported
            fi
            ;;
        "amazon_linux_ami")
            if [ ${DISTRO_MAJOR_VERSION} -lt 2016 ] || [ ${DISTRO_MAJOR_VERSION} -gt 2017 ]; then
                distro_not_supported
            fi
            ;;
        *)
            distro_not_supported
            ;;
    esac
}

_contact_us_for_help() 
{
    echo -e "Please contact us at ${BC}support@appoptics.com${EC} if you need assistance."
}

display_message_yesno()
{
    local message=${1}
    echo -ne "${YC}${message}? (Y/n)${EC}"
    if [ "$_SD_ASSUME_YES" = "" ]; then
        while :; do
            IFS= read -r _GET_INPUT
            case "$_GET_INPUT" in
                "Y"|"y"|"")
                    break
                    ;;
                "N"|"n")
                    echoyellow "OK, exiting."
                    echoyellow ""
                    exit 1
                    ;;
            esac
        done
    else # unattended install, assume we want to configure
        echo "y"
    fi
    echo ""
}

display_summary()
{
    echo -e  "${BC}${AGENT_DISPLAY_NAME} has been ${GC}succesfully${EC}${BC} configured.${EC}"
    echo     ""
    echo -e  " ${BC}================== ${GC} Configuration Summary ${BC}=======================${EC}"
    echo     ""
    echo -e  "   ${BC}The ${AGENT_DISPLAY_NAME} configuration file is ${GC}${APPOPTICS_CONF}.${EC}"
    echo     ""
    echo -e  "   ${BC}Auto load plugins configured in ${GC}${INSTALL_DIR}/autoload${EC}:"
    for file in ${DEFAULT_AUTOLOAD_DIR}/snap-plugin-*
    do
        if [[ -f $file ]]; then
            only_filename="${file##*/}"
            echo -e  "   ${BC} * ${GC}${only_filename}${EC}"
        fi
    done
    echo -e  "   ${BC}Auto load tasks configured in ${GC}${INSTALL_DIR}/autoload${EC}:"
    for file in ${DEFAULT_AUTOLOAD_DIR}/task*
    do
        if [[ -f $file ]]; then
            only_filename="${file##*/}"
            echo -e  "   ${BC} * ${GC}${only_filename}${EC}"
        fi
    done
    echo     ""
}

check_requirements()
{
    # Detect operating system
    detect_operating_system

    # Verify that distribution is supported
    check_distro_supported
}

usage() 
{
    cat << EOU

  Usage:  ${0} [script options]

  Script Options:

    -h, --help              Show this help screen.

    -g, --enable <plugin>   Auto loads default plugin. Copies default plugin  and associated task to ${DEFAULT_AUTOLOAD_DIR} directory.
    -i, --disable <plugin>  Removes default plugin and associated task from autoload ${DEFAULT_AUTOLOAD_DIR} directory.
    -l, --list              List default plugins and their status.

    --debug                 Show debug output
    --no-color              Disable color output
    -y, --yes               Assume yes to all questions (non-interactive).

  Examples:

     Options --enable and --disable can only be used for the default plugins comming with the package. 
     These options are not allowed for system plugin.

     For example to enable or disable the Docker plugin you could run the following commands. 
     
     Use the following command to enable Docker plugin:

     sudo ${0} --enable docker --yes

     Use the following command to disable Docker plugin:

     sudo ${0} --disable docker --yes

  Advanced Options:

    -p, --autoload-plugin <path-to-plugin> Auto loads plugin. Copies plugin to ${DEFAULT_AUTOLOAD_DIR} and ${DEFAULT_PLUGINS_DIR} directories.
    -c, --noautoload-plugin <plugin-name>  Removes plugin from autoload ${DEFAULT_AUTOLOAD_DIR} directory.

    -t, --autoload-task <path-to-task>     Auto loads task. Copies plugin to ${DEFAULT_AUTOLOAD_DIR} and ${DEFAULT_PLUGINS_DIR} directories.
    -e, --noautoload-task <task-name>      Removes task from autoload ${DEFAULT_AUTOLOAD_DIR} directory.

    -f, --download-plugin <http-plugin>    Downloads plugin file to ${DEFAULT_PLUGINS_DIR} directory.
    -b, --download-task <http-task>        Downloads task file to ${DEFAULT_TASKS_DIR} directory.

EOU
}

restart_service()
{
    echodebug "Restarting service ..."
    echodebug "Checking if system uses systemd or SysV(/etc/init.d) to manage the services."
    # Check if system uses systemd
    if [ -x "${SYSTEMCTL_BIN}" ]; then
        echodebug "Detected that system uses systemd to manage the services."

        # Stop service
        if [ "`${SYSTEMCTL_BIN} is-active ${SNAP_SERVICE_NAME}`" = "active" ]; then
            echoinfo "Stopping ${SNAP_SERVICE_NAME} service ..."
            ${SYSTEMCTL_BIN} stop ${SNAP_SERVICE_NAME}.service
            RETVAL=$?
            if [ ${RETVAL} -eq ${SUCCESS} ]; then
                echoinfo "${SNAP_SERVICE_NAME} service has STOPPED."
            else
                echoerror "Unable to stop ${SNAP_SERVICE_NAME} service. Service is already stopped."
                return ${FAILURE_SERVICE_UNABLE_TO_STOP}
            fi    
        fi

        # Start service
        echoinfo "Starting ${SNAP_SERVICE_NAME} service ..."
        ${SYSTEMCTL_BIN} start ${SNAP_SERVICE_NAME}.service
        RETVAL=$?
        if [ ${RETVAL} -eq ${SUCCESS} ]; then
            echoinfo "${SNAP_SERVICE_NAME} service has STARTED."
        else
            RETVAL=${FAILURE_SERVICE_NOT_RUNNING}
            echoerror ${RETVAL} "Unable to start ${SNAP_SERVICE_NAME} service. Service is NOT STARTED."
            return ${RETVAL}
        fi    

        # Wait for 10 seconds for service to be fully initialized.
        # Avoids case when service is running for a couple of seconds and fails because of configuration
        sleep 10

        # Check if service is running
        echoinfo "Checking if ${SNAP_SERVICE_NAME} service is running ..."
        if [ "`${SYSTEMCTL_BIN} is-active ${SNAP_SERVICE_NAME}`" = "active" ]; then
            echoinfo "${SNAP_SERVICE_NAME} service is RUNNING."
            RETVAL=${SUCCESS}
        else
            RETVAL=${FAILURE_SERVICE_NOT_RUNNING}
            echoerror ${RETVAL} "Unable to start ${SNAP_SERVICE_NAME} service. Service is NOT RUNNING."
            return ${RETVAL}
        fi
        return $RETVAL
    else
      # System uses SysV(/etc/init.d)
      echodebug "Detected that system uses SysV(/etc/init.d) to manage the services."
        
      echoinfo "Starting ${SNAP_SERVICE_NAME} service ..."
      # Start service
      if exists ${SERVICE_BIN} ; then
        # Stop service
        echoinfo "Stopping ${SNAP_SERVICE_NAME} service ..."
        ${SERVICE_BIN} ${SNAP_SERVICE_NAME} stop
        RETVAL=$?
        if [ ${RETVAL} -eq ${SUCCESS} ] ; then
            echoinfo "${SNAP_SERVICE_NAME} service is STOPPED."
        else
            warning "Unable to stop ${SNAP_SERVICE_NAME} service. Error: ${RETVAL}."
        fi

        # Start service
        echoinfo "Starting ${SNAP_SERVICE_NAME} service ..."
        ${SERVICE_BIN} ${SNAP_SERVICE_NAME} start
        RETVAL=$?
        if [ ${RETVAL} -eq ${SUCCESS} ] ; then
            echoinfo "${SNAP_SERVICE_NAME} service is STARTED."
        else
            RETVAL=${FAILURE_SERVICE_NOT_RUNNING}
            error ${RETVAL} "Unable to start ${SNAP_SERVICE_NAME} service. Service is NOT RUNNING."
        fi
      else
        RETVAL=${FAILURE_MISSING_EXECUTABLE}
        echoerror "Unable to start ${SNAP_SERVICE_NAME} service. Service ${SERVICE_BIN} command is missing."
      fi
        
      return $RETVAL
    fi
}

do_restart_service()
{
    restart_service
    RETVAL=$?
    if [ ${RETVAL} -eq ${SUCCESS} ] ; then
        echoinfo "${AGENT_DISPLAY_NAME} succesfully restarted using new configuration."
    else
        echoinfo "${AGENT_DISPLAY_NAME} faild to restart using new configuration."
        failed ${RETVAL}
    fi
    return ${RETVAL}
}

autoload_plugin()
{
    local path_file=${1}
    if [ -d ${path_file} ] ; then
        echoerror "${path_file} is a directory. Argument for --autoload-plugin should be path to a file."
        exit 1
    else
        if [ -f "${path_file}" ]; then
            local dir_file=${path_file%/*}
            local only_filename=${path_file##*/}

            # Create a copy inside default location for plugins if it is not already there
            if [ "${dir_file}" = "${DEFAULT_PLUGINS_DIR}" ] ; then
                echodebug "${path_file} is in ${DEFAULT_PLUGINS_DIR} default location for plugin files."
            else
                echodebug "${path_file} is not in ${DEFAULT_PLUGINS_DIR} default location for plugin files."

                echodebug "Copy ${path_file} to ${DEFAULT_PLUGINS_DIR} directory."
                cp ${path_file} ${DEFAULT_PLUGINS_DIR}

                # Check that file was copied
                if [ -f "${DEFAULT_PLUGIN_DIR}/${only_filename}" ] ; then
                    echoinfo "${DEFAULT_PLUGIN_DIR}/${only_filename} copied."
                else
                    echoerror "Unable to copy ${path_file} to ${DEFAULT_PLUGINS_DIR} directory."
                    exit 1
                fi
            fi

            # Delete symlink if already exists
            if [ -e "${DEFAULT_AUTOLOAD_DIR}/${only_filename}" ] ; then
                echodebug "Unlink existing ${DEFAULT_AUTOLOAD_DIR}/${only_filename} link."
                rm ${DEFAULT_AUTOLOAD_DIR}/${only_filename}
            fi

            echoinfo "Creating symlink ${DEFAULT_AUTOLOAD_DIR}/${only_filename} to ${DEFAULT_PLUGINS_DIR}/${only_filename} ..."
            ln -s ${DEFAULT_PLUGINS_DIR}/${only_filename} ${DEFAULT_AUTOLOAD_DIR}/${only_filename}

            # Check that symlink was created
            if [ -e "${DEFAULT_AUTOLOAD_DIR}/${only_filename}" ] ; then
                echoinfo "${DEFAULT_AUTOLOAD_DIR}/${only_filename} created."
            else
                echodebug "Unable to create symlink ${DEFAULT_AUTOLOAD_DIR}/${only_filename} to ${DEFAULT_PLUGINS_DIR}/${only_filename}."
                exit 1
            fi
        else
            echoerror "${path_file} is a not a valid file. Argument for --autoload-plugin should be path to a file."
            exit 1
        fi
    fi
}

unload_plugin()
{
    local plugin_name=${1}
    local path_file=${DEFAULT_AUTOLOAD_DIR}/${plugin_name}

    if [ -f "${path_file}" ]; then
        echoinfo "Removing ${path_file} ..."
        rm ${path_file}

        # Check that symlink was removed
        if [ -L "${DEFAULT_AUTOLOAD_DIR}/${only_filename}" ] ; then
            echoerror "Unable to remove ${path_file}."
            exit 1
        else
            echoinfo "${path_file} plugin removed."
        fi
    else
        echoerror "Unable to find a plugin ${path_file}. Argument for --noautoload-plugin should be a plugin filename in ${DEFAULT_AUTOLOAD_DIR}."
        exit 1
    fi
}

unload_task()
{
    local task_name=${1}
    local path_file=${DEFAULT_AUTOLOAD_DIR}/${task_name}.${DEFAULT_TASK_EXTENSION}

    if [ -f "${path_file}" ]; then
        echoinfo "Removing ${path_file} ..."
        rm ${path_file}

        # Check that symlink was removed
        if [ -e "${path_file}" ] ; then
            echoerror "Unable to remove ${path_file}."
            exit 1
        else
            echoinfo "${path_file} task removed."
        fi
    else
        echoerror "Unable to find a task ${path_file}. Argument for --nounload-task should be a task name in ${DEFAULT_AUTOLOAD_DIR}."
        exit 1
    fi
}

autoload_task()
{
    local path_file=${1}
    if [ -d ${path_file} ] ; then
        echoerror "${path_file} is a directory. Argument for --autoload-task should be path to a file."
        exit 1
    else
        if [ -f "${path_file}" ]; then
            local dir_file=${path_file%/*}
            local only_filename=${path_file##*/}

            # Create a copy inside default location for tasks if not already there
            if [ "${dir_file}" = "${DEFAULT_TASKS_DIR}" ] ; then
                echodebug "${path_file} is in ${DEFAULT_TASKS_DIR} default location for tasks files."
            else
                echodebug "${path_file} is not in ${DEFAULT_TASKS_DIR} default location for tasks files."

                echoinfo "Copying ${path_file} to ${DEFAULT_TASKS_DIR} ..."
                cp ${path_file} ${DEFAULT_TASKS_DIR}

                # Check that file was copied
                if [ -f "${DEFAULT_TASKS_DIR}/${only_filename}" ] ; then
                    echoinfo "${DEFAULT_TASKS_DIR}/${only_filename} copied."
                else
                    echoerror "Unable to copy ${path_file} to ${DEFAULT_TASKS_DIR} directory."
                    exit 1
                fi
            fi

            # Delete symlink if already exists
            if [ -e ${DEFAULT_AUTOLOAD_DIR}/${only_filename} ] ; then
                echodebug "Unlink existing ${DEFAULT_AUTOLOAD_DIR}/${only_filename} link."
                rm ${DEFAULT_AUTOLOAD_DIR}/${only_filename}
            fi

            echoinfo "Creating symlink ${DEFAULT_AUTOLOAD_DIR}/${only_filename} to ${DEFAULT_TASKS_DIR}/${only_filename} ..."
            ln -s ${DEFAULT_TASKS_DIR}/${only_filename} ${DEFAULT_AUTOLOAD_DIR}/${only_filename}

            # Check that symlink was created
            if [ -e "${DEFAULT_AUTOLOAD_DIR}/${only_filename}" ] ; then
                echoinfo "${DEFAULT_AUTOLOAD_DIR}/${only_filename} created."
            else
                echoerror "Unable to create symlink ${DEFAULT_AUTOLOAD_DIR}/${only_filename} to ${DEFAULT_TASKS_DIR}/${only_filename}."
                exit 1
            fi
        else
            echoerror "${path_file} is a not a valid file. Argument for --autoload-plugin should be path to a file."
            exit 1
        fi
    fi
}

download_plugin()
{
    local url=${1}
    __create_temp_directory
    plugin_name=${url##*/}
    __fetch_url ${_SD_TMP_DIR}/${plugin_name} ${url}
    local files=( ${_SD_TMP_DIR}/${plugin_name}* )
    for file in "${files[@]}"
    do
       only_filename="${file##*/}"
       if [ -f "${DEFAULT_PLUGINS_DIR}/${only_filename}" ] ; then
           echowarn "Plugin ${only_filename} already present in ${DEFAULT_PLUGINS_DIR} directory."
           display_message_yesno "Do you want to overwrite ${only_filename}"
       fi

       echoinfo "Copying ${file} to ${DEFAULT_PLUGINS_DIR} directory ..."
       cp ${file} ${DEFAULT_PLUGINS_DIR}

       # Check that file was copied
       if [ -f "${DEFAULT_PLUGINS_DIR}/${only_filename}" ] ; then
            echoinfo "${DEFAULT_PLUGINS_DIR}/${only_filename} copied."
       else
            echoerror "Unable to copy ${file} to ${DEFAULT_PLUGINS_DIR} directory."
            exit 1
       fi
    done
}

download_task()
{
    local url=${1}
    __create_temp_directory
    task_name=${url##*/}
    __fetch_url ${_SD_TMP_DIR}/${task_name} ${url}
    local files=( ${_SD_TMP_DIR}/${task_name}* )
    for file in "${files[@]}"
    do
       only_filename="${file##*/}"
       if [ -f "${DEFAULT_TASKS_DIR}/${only_filename}" ] ; then
           echowarn "Task ${only_filename} already present in ${DEFAULT_TASKS_DIR} directory."
           display_message_yesno "Do you want to overwrite ${only_filename}"
       fi

       echoinfo "Coping ${file} to ${DEFAULT_TASKS_DIR} directory ..."
       cp ${file} ${DEFAULT_TASKS_DIR}

       # Check that file was copied
       if [ -f "${DEFAULT_TASKS_DIR}/${only_filename}" ] ; then
           echoinfo "${DEFAULT_TASKS_DIR}/${only_filename} copied."
       else
           echoerror "Unable to copy ${file} to ${DEFAULT_TASKS_DIR} directory."
           exit 1
       fi
    done
}

check_docker_requirements()
{
  local docker_group_name=docker
  if getent group ${docker_group_name} >/dev/null; then
      echodebug "Found ${docker_group_name} group. Adding ${APPOPTICS_USERNAME} to ${docker_group_name} group ..."
      usermod -aG ${docker_group_name} ${APPOPTICS_USERNAME}
      RETVAL=$?
      if [ $RETVAL -ne 0 ]; then
          echoerror "Unable to add ${APPOPTICS_USERNAME} user to ${docker_group_name} group. Error code: ${RETVAL}."
          exit 1
      else
          echodebug "${APPOPTICS_USERNAME} added to ${docker_group_name} group."
      fi
 else
     echowarn "Unable to find ${docker_group_name} group. Docker plugin will not work without ${APPOPTICS_USERNAME} user being added to
     ${docker_group_name} group."
     exit 1
 fi
}

display_message_configure_plugin()
{
    echo -e ""
    echo -e "${BC}To configure the plugin you need to create a copy of the ${YC}plugin.yaml.example${BC} sample${EC}"
    echo -e "${BC}configuration file to ${YC}/opt/appoptics/etc/plugin.d${BC} directory, add specific configuration${EC}" 
    echo -e "${BC}required by plugin and run ${YC}–enable${BC} plugin command again.${EC}"
    echo -e ""
}

enable_plugin()
{
    local plugin_short_name=${1}
    local plugin_name=${default_plugins_long_names[${plugin_short_name}]}
    local plugin_config_filename=${default_plugins_configuration_file[${plugin_short_name}]}
    local plugin_config_file=${DEFAULT_PLUGINS_CONF_DIR}/${plugin_config_filename}

    local task_name=${default_plugins_tasks[${plugin_short_name}]}
    local task_filename=${task_name}.${DEFAULT_TASK_EXTENSION}
    local task_path=${DEFAULT_TASKS_DIR}/${task_filename}

    if [ -z "${plugin_name}" ] ; then
        echoerror "Unknown default plugin ${plugin_short_name}."
        exit 1
    fi

    if [ "${plugin_short_name}" = "system" ] ; then
        echoerror "Enable option is not allowed for plugin ${plugin_short_name}(${plugin_name})."
        exit 1
    fi

    if [ -f "${plugin_config_file}" ] ; then
        echodebug "Found plugin ${plugin_config_file} configuration file. User has configured plugin."
    else
        echoerror "Unable to find ${plugin_config_file} configuration file. Plugin configuration file must be present in ${DEFAULT_PLUGINS_CONF_DIR} directory before enabling ${plugin_short_name}(${plugin_name}) plugin."
        display_message_configure_plugin
        exit 1
    fi

    if [ "${plugin_short_name}" = "docker" ] ; then
        check_docker_requirements
    fi

    local has_changed=${__IS_FALSE}

    if [ -f "${DEFAULT_PLUGINS_DIR}/${plugin_name}" ] ; then
        echodebug "Enabling ${plugin_short_name}(${plugin_name}) plugin ..."
        autoload_plugin ${DEFAULT_PLUGINS_DIR}/${plugin_name}
        echodebug "${plugin_short_name}(${plugin_name}) enabled."
        has_changed=${__IS_TRUE}
    else
        echoerror "Unable to find ${DEFAULT_PLUGINS_DIR}/${plugin_name}."
        exit 1
    fi

    if [ -z "${task_name}" ] ; then
       echoerror "Unknown default task ${task_name}."
    else
        if [ -f "${task_path}" ] ; then
            echodebug "Enabling ${task_name} task ..."
            autoload_task ${task_path}
            echodebug "${task_name} enabled."
            has_changed=${__IS_TRUE}
        else
            echoerror "Unable to find ${task_path}."
            exit 1
        fi
    fi

    if [ ${has_changed} -eq ${__IS_TRUE} ] ; then
        echoinfo "${plugin_name} ENABLED."
    fi
}

disable_plugin()
{
    local plugin_short_name=${1}
    local plugin_name=${default_plugins_long_names[${plugin_short_name}]}

    local task_name=${default_plugins_tasks[${plugin_short_name}]}
    local task_filename=${task_name}.${DEFAULT_TASK_EXTENSION}
    local task_path=${DEFAULT_AUTOLOAD_DIR}/${task_filename}

    if [ -z "${plugin_name}" ] ; then
        echoerror "Unknown default plugin ${plugin_short_name}."
        exit 1
    fi

    if [ "${plugin_short_name}" = "system" ] ; then
        echoerror "Disable option is not allowed for plugin ${plugin_short_name}(${plugin_name})."
        exit 1
    fi

    local has_changed=${__IS_FALSE}

    if [ -f "${DEFAULT_AUTOLOAD_DIR}/${plugin_name}" ] ; then
        echodebug "Disabling ${plugin_short_name}(${plugin_name}) ..."
        unload_plugin ${plugin_name}
        echodebug "${plugin_short_name}(${plugin_name}) disabled."
        has_changed=${__IS_TRUE}
    else
        echodebug "Unable to find ${DEFAULT_AUTOLOAD_DIR}/${plugin_name}."
    fi

    if [ -z "${task_name}" ] ; then
       echoerror "Unknown default task ${task_name}."
    else
        if [ -f "${task_path}" ] ; then
            echodebug "Disabling ${task_name} task ..."
            unload_task ${task_name}
            echodebug "${task_name} disabled."
            has_changed=${__IS_TRUE}
        else
            echodebug "Unable to find ${task_path}."
        fi
    fi

    if [ ${has_changed} -eq ${__IS_TRUE} ] ; then
        echoinfo "${plugin_name} DISABLED."
    fi
}

list_plugins_status()
{
    echo -e  "${BC} Auto load configuration for default plugins:${EC}"
    echo     ""
    for plugin_short_name in ${!default_plugins_long_names[@]}; do
        local plugin_name=${default_plugins_long_names[${plugin_short_name}]}
        local task_name=${default_plugins_tasks[${plugin_short_name}]}
        local task_filename=${task_name}.${DEFAULT_TASK_EXTENSION}

        local task_path=${DEFAULT_AUTOLOAD_DIR}/${task_filename}
        local plugin_path=${DEFAULT_AUTOLOAD_DIR}/${plugin_name}

        local is_plugin_autoloaded=${__IS_FALSE}
        if [ -e "${plugin_path}" ] ; then
            is_plugin_autoloaded=${__IS_TRUE}
        fi

        local is_task_autoloaded=${__IS_FALSE}
        if [ -e "${task_path}" ] ; then
            is_task_autoloaded=${__IS_TRUE}
        fi

        echo -e  "${BC} * ${GC}${plugin_short_name} (${plugin_name})${BC}: ${EC}"
        if [ ${is_plugin_autoloaded} -eq ${__IS_TRUE} ] ; then
            echo -e  "  ${BC} * plugin is ${GC}autoloaded${BC}.${EC}"
        else
            echo -e  "  ${BC} * plugin is ${YC}not autoloaded${BC}.${EC}"
        fi

        if [ ${is_task_autoloaded} -eq ${__IS_TRUE} ] ; then
            echo -e  "  ${BC} * ${task_filename} is ${GC}autoloaded${BC}.${EC}"
        else
            echo -e  "  ${BC} * ${task_filename} is ${YC}not autoloaded${BC}.${EC}"
        fi
        echo ""
    done
}

do_agent_configuration()
{
    local config_has_changed=${__IS_FALSE}

    if [ -n "${DOWNLOAD_PLUGIN_FILE+set}" ]; then
        download_plugin ${DOWNLOAD_PLUGIN_FILE}
    fi

    if [ -n "${DOWNLOAD_TASK_FILE+set}" ]; then
        download_task ${DOWNLOAD_TASK_FILE}
    fi

    if [ -n "${AUTOLOAD_PLUGIN_FILE+set}" ]; then
        autoload_plugin ${AUTOLOAD_PLUGIN_FILE}
        config_has_changed=1
    fi

    if [ -n "${AUTOLOAD_TASK_FILE+set}" ]; then
        autoload_task ${AUTOLOAD_TASK_FILE}
        config_has_changed=1
    fi

    if [ -n "${NOAUTOLOAD_PLUGIN_FILE+set}" ]; then
        unload_plugin ${NOAUTOLOAD_PLUGIN_FILE}
        config_has_changed=1
    fi

    if [ -n "${NOAUTOLOAD_TASK_FILE+set}" ]; then
        unload_task ${NOAUTOLOAD_TASK_FILE}
        config_has_changed=1
    fi

    if [ -n "${ENABLE_PLUGIN_NAME+set}" ]; then
        enable_plugin ${ENABLE_PLUGIN_NAME}
        config_has_changed=1
    fi

    if [ -n "${DISABLE_PLUGIN_NAME+set}" ]; then
        disable_plugin ${DISABLE_PLUGIN_NAME}
        config_has_changed=1
    fi

    if [ -n "${_ECHO_LIST_DEFAULT_PLUGIN_STATUS+set}" ]; then
        _SD_ASSUME_YES=${__IS_TRUE}
        list_plugins_status
    fi

    # Restart service only when configuration has changed
    if [ ${config_has_changed} -eq ${__IS_TRUE}  ]; then
        do_restart_service
    fi

    display_summary
}

_ORIG_OPTS=$@
while :; do
    if [ $# -eq 0 ]; then break; fi
    case ${1} in
        -h|--help)
            usage
            exit
            ;;
        -p|--autoload-plugin)
            AUTOLOAD_PLUGIN_FILE=${2}
            shift 2
            continue
            ;;
        -t|--autoload-task)
            AUTOLOAD_TASK_FILE=${2}
            shift 2
            continue
            ;;
        -c|--noautoload-plugin)
            NOAUTOLOAD_PLUGIN_FILE=${2}
            shift 2
            continue
            ;;
        -e|--noautoload-task)
            NOAUTOLOAD_TASK_FILE=${2}
            shift 2
            continue
            ;;
        -f|--download-plugin)
            DOWNLOAD_PLUGIN_FILE=${2}
            shift 2
            continue
            ;;
        -b|--download-task)
            DOWNLOAD_TASK_FILE=${2}
            shift 2
            continue
            ;;
         -g|--enable )
            ENABLE_PLUGIN_NAME=${2}
            shift 2
            continue
            ;;
         -i|--disable )
            DISABLE_PLUGIN_NAME=${2}
            shift 2
            continue
            ;;
         -l|--list)
            _ECHO_LIST_DEFAULT_PLUGIN_STATUS=${__IS_TRUE}
           ;;
         -D|--debug)
            _ECHO_DEBUG=${__IS_TRUE}
           ;;
         --no-color)
            # This is already detected at the very beginning of the script,
            # but we have to detect this as a valid script option otherwise
            # it will be caught in the *) case and cause the option loop to exit
            _SD_NO_COLOR=${__IS_TRUE}
            ;;
        -y|--yes)
            _SD_ASSUME_YES=${__IS_TRUE}
            ;;
        *)
            break
            ;;
    esac
    shift
done

if [ "$BASH_VERSION" = "" ]; then
    __bash_path=$(which bash)
    echoerror "This script must run under bash."
    if [ "$__bash_path" != "" ]; then
        echoerror "Try running: $__bash_path $0 $@"
    fi
    exit 1
fi

if [ $? -ne 0 ] && [ "$_SD_ASSUME_YES" = "" ]; then
    echoerror "No tty detected. Specify the -y option to run non-interactively."
    echoerror "Running remotely via SSH? Allocate a pseudo TTY using \"ssh -t ...\""
    usage
    exit 1
fi

if ((${EUID:-0} || "$(id -u)")); then
  echoerror "You need to be root to run this script. Try running: sudo $0 $@"
  exit 1
fi

echoblue ""
echo -e  " ${BC}======================${GC} ${AGENT_DISPLAY_NAME} Configuration ${BC}======================="
echoblue "  This script will walk you through the configuration of ${AGENT_DISPLAY_NAME}."
echoblue ""
echoblue "  For detailed instructions, FAQ, and full documentation:"
echoyellow "   http://docs.appoptics.com/kb/host_infra/host_agent"
echoblue ""
echoblue "  If you have any questions or problems, please contact us:"
echoyellow "   email: support@appoptics.com"
echoblue " ==============================================================="
echo     ""

display_message_yesno "Are you ready to configure ${AGENT_DISPLAY_NAME}"

check_requirements

_AGENT_INSTALLED=$(__is_package_installed "${PACKAGE_NAME}")
if [ $_AGENT_INSTALLED -eq $__IS_TRUE ]; then
    do_agent_configuration
else
    echoerror "${AGENT_DISPLAY_NAME} is not installed."
    exit 1
fi


